local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:OnInitialize()
	local defaults = {
			profile = {
				version = 0.11,
			
				margin = 100,
				minMargin = 100*10000,
				maxMargin = 300*10000,
				minProfit = 0*10000,
				showByDefault = false,
				cancelYellow = true,
				cancelBlue = true,
				cancelDuration = true,
				filters = {
					global = {
						Northrend = true,
						BurningCrusade = false,
						Classic = false,
					},
				},
			}
		}
	self.db = LibStub("AceDB-3.0"):New("CapitalistDB", defaults, true)

	if (not self.db.profile.components) then
		self.db.profile.components = {}
	end
	if (not self.db.profile.professions) then
		self.db.profile.professions = {}
	end
	if (not self.db.profile.storage) then
		self.db.profile.storage = {}
	end
	if (not self.db.profile.debug) then
		self.db.profile.debug = {}
	end
	
	if (not self.db.profile.storage[realm]) then
		self.db.profile.storage[realm] = {}
	end
	
	if (not self.db.profile.storage[realm][character]) then
		self.db.profile.storage[realm][character] = { AH = {}, bank = {}, inventory = {} }
	end
	
	self:DebugMisc("Profile Version: "..self.db.profile.version)
	self:DebugMisc("Current Version: "..defaults.profile.version)
	
	self.db.profile.version = defaults.profile.version

	-- Late DebugEntry so that self.db is prepared
	self:DebugEntry("OnInitialize")

	self.updateHandler = CreateFrame("frame")
	self.updateHandler:Hide()
	self.elapsed = 0
	self.updateHandler:SetScript("OnUpdate", function(self, elapsed)
												self = Capitalist
--												self:DebugEntry("OnUpdate")
												self.elapsed = self.elapsed + elapsed
--												self:DebugMisc(self.mode)
												
												if (self.mode == "scanning") then
													self:OnUpdateScanning()
												elseif (self.mode == "posting") then
													self:OnUpdatePosting()
												elseif (self.mode == "componentsearching") then
													self:OnUpdateComponentSearching()
												else
													self:OnUpdateComponentScanning()
												end
--												self:DebugExit("OnUpdate")
											end)
	
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED")
	self:RegisterEvent("AUCTION_HOUSE_SHOW")
	self:RegisterEvent("AUCTION_ITEM_LIST_UPDATE")
	self:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("MERCHANT_CLOSED")
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("UI_ERROR_MESSAGE")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	
	if (IsAddOnLoaded("Blizzard_AuctionUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_AuctionUI")
	end
	if (IsAddOnLoaded("Blizzard_TradeSkillUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_TradeSkillUI")
	end


	self:ResetSkillCaches()

	self:EnforceProductsCannotHaveCostSetting()
	self:UpdateComponentValues()
	self:UpdateCombinedValues()

	self:BuildOptions()
	self:ApplyFilters()

	self:DebugExit("OnInitialize")
end

function Capitalist:ADDON_LOADED(event, addon)
	self:DebugEntry("ADDON_LOADED")

	if (addon == "Blizzard_AuctionUI") then
		self:CreateAuctionUI()
		self:CreateComponentUI()
		if (self.db.profile.showByDefault) then
			self:ToggleAuctionUI()
		end
	end
	if (addon == "Blizzard_TradeSkillUI") then
		self.restockButton = CreateFrame("button", "CapitalistRestockButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.restockButton:SetPoint("TOPRIGHT", TradeSkillFrame, "TOPRIGHT", -25, -2)
		self.restockButton:SetHeight(17)
		self.restockButton:SetWidth(60)
		self.restockButton:SetText("(C)")
		self.restockButton:SetScript("OnClick", function() Capitalist:Restock(CURRENT_TRADESKILL) end)
		self.restockButton:Show()

		self.showStackButton = CreateFrame("button", "CapitalistShowStackButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.showStackButton:SetPoint("TOPRIGHT", self.restockButton, "TOPLEFT", 0, 0)
		self.showStackButton:SetHeight(17)
		self.showStackButton:SetWidth(30)
		self.showStackButton:SetText("(R)")
		self.showStackButton:SetScript("OnClick", function() Capitalist:ShowRestockStack(CURRENT_TRADESKILL) end)
		self.showStackButton:Show()
		
		self.TradeSkillFrame_Show_Hook = TradeSkillFrame_Show
		TradeSkillFrame_Show =  function()
									self:DebugMisc(CURRENT_TRADESKILL)
									Capitalist.TradeSkillFrame_Show_Hook()
									self:ScanTradeSkillFrame(CURRENT_TRADESKILL)
								end
									
	end

	self:DebugExit("ADDON_LOADED")
end

function Capitalist:AUCTION_ITEM_LIST_UPDATE()
	self:DebugEntry("AUCTION_ITEM_LIST_UPDATE")

	if (self.mode == "scanning") then
		if (not self.ScanAnalyzed) then
			self:OnAuctionItemListUpdate()
		end
	end

	self:DebugExit("AUCTION_ITEM_LIST_UPDATE")
end

function Capitalist:AUCTION_HOUSE_CLOSED()
	self:DebugEntry("AUCTION_HOUSE_CLOSED")

	if (self.updateHandler:IsVisible()) then
		self.updateHandler:Hide()
		
		if (self.mode == "scanning") then
			self.auctionUI.lastScanLabel:SetText(L["Last Scan: failed"])
			self.auctionUI.lastScanLabel:Show()
			self.auctionUI.progress:Hide()
			self.auctionUI.progressLabel:Hide()
			self.auctionUI.scanButton:Enable()
			self.validScan = false
		end
		if (self.mode == "posting") then
			self.auctionUI.lastScanLabel:Show()
			self.auctionUI.progress:Hide()
			self.auctionUI.progressLabel:Hide()
			self.auctionUI.scanButton:Enable()
			self.auctionUI.postButton:Enable()
		end
	end

	self:DebugExit("AUCTION_HOUSE_CLOSED")
end

function Capitalist:HandleSlashCommand(...)
	self:DebugEntry("HandleSlashCommand")

	--local arg1, arg2, arg3, arg4, arg5, arg6 = ...
	local arg1 = ...
	self:DebugMisc(arg1)

	if ((arg1 == "?") or (arg1 == "h") or (arg1 == "help")) then
		self:ReportSyntax()
	elseif (string.sub(arg1, 1, 2) == "c ") then
		self:OutputItemCost(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 5) == "cost ") then
		self:OutputItemCost(arg1.sub(arg1, 6))
	elseif (string.sub(arg1, 1, 2) == "f ") then
		self:FindProducts(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 5) == "find ") then
		self:FindProducts(arg1.sub(arg1, 6))
	elseif ((arg1 == "m") or (arg1 == "missing")) then
		self:OutputMissingCosts()
	elseif ((arg1 == "p") or (arg1 == "professions")) then
		if (self.db.profile[realm] and
			self.db.profile[realm][character]) then

			for profession, professionData in pairs(self.db.profile[realm][character]) do
				self:Print(profession)
			end
		end
	elseif ((arg1 == "r") or (arg1 == "restock")) then
		if (CURRENT_TRADESKILL) then
			self:Restock(CURRENT_TRADESKILL)
		end	
	elseif (string.sub(arg1, 1, 2) == "s ") then
		self:OutputProductCraftingSequence(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 9) == "sequence ") then
		self:OutputProductCraftingSequence(arg1.sub(arg1, 10))
	elseif ((arg1 == "t") or (arg1 == "top10")) then
		self:ReportTop10()
	elseif ((arg1 == "v") or (arg1 == "version")) then
		self:ReportVersion()
	else
		InterfaceOptionsFrame_OpenToCategory("Capitalist")
	end

	self:DebugExit("HandleSlashCommand")
end
