local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:BuildOptions()
	self:DebugEntry("BuildOptions")

	self.options = {
		type = "group",
		args = {
			general = {
				name = L["General"],
				type = "group",
				order = 1,
				args = {
					minProfit = {
						name = L["Minimal profit"],
						order = 1,
						desc = L["How much you want to gain at minimum from each product (price is simply added to material cost)."],
						type = "input",
						width = "half",
						set = function(info, val)
								self.db.profile.minProfit = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minProfit / 10000
							  end,
					},
					margin = {
						name = L["Price margin"],
						order = 2,
						desc = L["The increase (in percent) in price, if there is no or very expensive competition."],
						type = "input",
						width = "half",
						set = function(info, val)
								self.db.profile.margin = tonumber(val) or 0
							  end,
						get = function(info)
								return ""..self.db.profile.margin
							  end,
					},
					minMargin = {
						name = L["Minimal increase"],
						order = 3,
						desc = L["How much (in gold) the price should increase at minimum if there is no or very expensive competition."],
						type = "input",
						width = "half",
						set = function(info, val)
								self.db.profile.minMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minMargin / 10000
							  end,
					},
					maxMargin = {
						name = L["Maximal increase"],
						order = 4,
						desc = L["How much (in gold) the price should increase at maximum if there is no or very expensive competition."],
						type = "input",
						width = "half",
						set = function(info, val)
								self.db.profile.maxMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.maxMargin / 10000
							  end,
					},
					showByDefault = {
						name = L["Show by default"],
						order = 5,
						desc = L["Automatically opens Capitalist when you enter the Auction House"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.showByDefault = val
							  end,
						get = function(info)
								return self.db.profile.showByDefault
							  end,
					},
					merchantCosting = {
						name = L["Merchant costing"],
						order = 6,
						desc = L["When unlimited items are found on a merchant, update component price to match."],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.enableMerchantCosting = true
								else
									self.db.profile.enableMerchantCosting = nil
								end
							  end,
						get = function(info)
								return self.db.profile.enableMerchantCosting
							  end,
					},
					bidAtCost = {
						name = L["Bid price at cost"],
						order = 7,
						desc = L["When posting, set the bid price at cost (default is buyout price)."],
						type = "toggle",
						set = function(info, val)
								if (val) then
									self.db.profile.enableBidAtCost = true
								else
									self.db.profile.enableBidAtCost = nil
								end
							  end,
						get = function(info)
								return self.db.profile.enableBidAtCost
							  end,
					},
					headerExpansionFilters = {
						name = "Expansion filters",
						order = 20,
						type = "header",
					},
					Northrend = {
						name = L["Northrend"],
						order = 21,
						desc = L["If checked, products associated with Northrend will be produced."],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Northrend = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Northrend
							  end,
					},
					BurningCrusade = {
						name = L["Burning Crusade"],
						order = 22,
						desc = L["If checked, products associated with Burning Crusade will be produced."],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.BurningCrusade = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.BurningCrusade
							  end,
					},
					Classic = {
						name = L["Classic"],
						order = 23,
						desc = L["If checked, products associated with Classic will be produced."],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Classic = val
								self:ApplyFilters()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Classic
							  end,
					},
				}
			},
			resourceCosts = {
				name = L["Resource Costs"],
				type = "group",
				order = 81,
				args = {}
			},
			createdProducts = {
				name = L["Created Products"],
				type = "group",
				order = 82,
				args = {}
			},
			data = {
				name = L["Data Management"],
				type = "group",
				order = 91,
				args = {
					ClearMerchants = {
						name = L["Clear merchant data"],
						order = 1,
						desc = L["Clear merchant name and buy price for merchant-provided with resources."],
						type = "execute",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											if (self:ResourceGetMerchant(componentName)) then
												self:DebugMisc(componentName)
												self:ResourceSetMerchant(componentName, nil)
												self:SetResourceCopperBuyPrice(componentName, nil)
											end
										end
									end
								end,
					},
					ClearBuyPrices = {
						name = L["Clear user buy prices"],
						order = 2,
						desc = L["Clear buy prices set by the user."],
						type = "execute",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											if (not self:ResourceGetMerchant(componentName)) then
												self:DebugMisc(componentName)
												self:SetResourceCopperBuyPrice(componentName, nil)
											end
										end
									end
								end,
					},
					ClearCosts = {
						name = L["Clear costs"],
						order = 3,
						desc = L["Clear user-set resource costs."],
						type = "execute",
						func = function()
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											self:DebugMisc(componentName)
											self:SetResourceCopperCost(componentName, nil)
										end
									end
								end,
					},
					ClearResourceValues = {
						name = L["Clear resource values"],
						order = 4,
						desc = L["Clear calculated values of resources."],
						type = "execute",
						func = function()
									self:LogClearCategory("value")
									if (self.db.profile.components) then
										for componentName, componentCount in pairs(self.db.profile.components) do
											self:DebugMisc(componentName)
											self:SetComponentCopperValue(componentName, nil)
										end
									end
								end,
					},
				},
			},
			debug = {
				name = L["Debug"],
				type = "group",
				order = 92,
				args = {
					TraceFunctions = {
						name = "Trace function entry/exit",
						order = 1,
						desc = "Output debug text on entry and exit of functions that support it.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceFunctions = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceFunctions
							  end,
					},
					TraceMisc = {
						name = "Trace misc. state",
						order = 2,
						desc = "Output extra debug information scattered throughout.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceMisc = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceMisc
							  end,
					},
					LogValueCalc = {
						name = "Log value calculations",
						order = 3,
						desc = "Record value calculations in log field of SavedVariables.",
						type = "toggle",
						set = function(info, val)
									self.db.profile.debug.LogValueCalc = val
								end,
						get = function(info)
									return self.db.profile.debug.LogValueCalc
								end,
					},
					Calculate = {
						name = "Recalc",
						order = 4,
						desc = "Recalculate values.",
						type = "execute",
						func = function()

									print("UpdateComponentValues")
									if (self:UpdateComponentValues()) then
										print("  delta")
									end

									print("UpdateCombinedValues")
									if (self:UpdateCombinedValues()) then
										print("  delta")
									end
								end,
					},
				}
			},
		}
	}

	-- REVIEW: Add Debug options to clear out professions, components, learned products, etc.
	-- ........BUT keep the counts and filters.

	-- add profession nodes
	local order = 11

	local knownProfessions = {}

	if (self.db.profile.professions) then
		for profession, professionData in pairs(self.db.profile.professions) do
			table.insert(knownProfessions, profession)
		end
	end
	table.sort(knownProfessions)

	for i=1, #knownProfessions do
		local profession = knownProfessions[i]
		self.options.args[profession] = {
											name = profession,
											type = "group",
											order = order,
											args = {
												filters = {
													name = L["Filters"],
													type = "group",
													order = 1,
													args = {}
												},
												counts = {
													name = L["Counts"],
													type = "group",
													order = 2,
													args = {}
												},
											}
										}

		order = order + 1

	end	

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Capitalist", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Capitalist")
	self:RegisterChatCommand("Capitalist", function(...) Capitalist:HandleSlashCommand(...) end)
	self:RegisterChatCommand("cap", function(...) Capitalist:HandleSlashCommand(...) end)

	self:DebugExit("BuildOptions")
end

function Capitalist:UpdateProductOptions()
	self:DebugEntry("UpdateProductOptions")

	if (self.db.profile.professions) then
		for profession, professionData in pairs(self.db.profile.professions) do
			if (self.options.args[profession]) then
				self:DebugMisc(profession)

				local t

				-- populate filters
				self.options.args[profession].args.filters.args = {}
				t = self.options.args[profession].args.filters.args

				for skillName, skillData in pairs(professionData) do
					local numCreated = skillData.numCreated or 0

					local product = skillData
					self:DebugMisc(skillName)
					self:DebugMisc(product)
	
					if (self:ProductIsVisible(profession, skillName)) then

						local settingSkillName = L.SettingUIFromSkillName(profession, skillName)
						t[skillName] = {
							name = settingSkillName,
							desc = product.description,
							type = "toggle",
							set = function(info, val)
									self:ProductSetEnabled(profession, skillName, val)
									self:ApplyFilters()
									end,
							get = function(info)
									return self:ProductIsEnabled(profession, skillName)
									end
						}
					end
				end
	
				t["all"] = {
					name = L["all products"],
					desc = L["enable or disable all products"],
					type = "toggle",
					order = 1,
					tristate = true,
					set = function(info, val)
								for skillName, skillData in pairs(professionData) do
									if (self:ProductIsVisible(profession, skillName)) then
										self:ProductSetEnabled(profession, skillName, val)
									end
								end
								self:ApplyFilters()
							end,
					get = function(info)
								local found
								for skillName, skillData in pairs(professionData) do
									if (self:ProductIsVisible(profession, skillName)) then
										local productEnabled = self:ProductIsEnabled(profession, skillName)
										if ((found ~= nil) and (found ~= productEnabled)) then
											return nil
										else
											found = productEnabled
										end
									end
								end
								return found
							end
				}

				-- populate product count
				self.options.args[profession].args.counts.args = {}
				t = self.options.args[profession].args.counts.args

				for skillName, skillData in pairs(professionData) do

					if (self.db.profile.professions[profession] and
						self.db.profile.professions[profession][skillName]) then

						local product = self.db.profile.professions[profession][skillName]
						self:DebugMisc(skillName)
						self:DebugMisc(product)

						if (self:ProductIsEnabled(profession, skillName) and
							self:ProductIsVisible(profession, skillName)) then

							local settingSkillName = L.SettingUIFromSkillName(profession, skillName)

							t[skillName] = {
								name = settingSkillName,
								desc = product.description,
								type = "input",
								pattern = "^%d$",
								set = function(info, val)
											local nVal = tonumber(val) or 0
											self:ProductSetCount(profession, skillName, nVal)
										end,
								get = function(info, val)
											return ""..self:ProductGetCount(profession, skillName)
										end
							}
						end
					end
				end
			end
		end
	end

	self:DebugExit("UpdateProductOptions")
end

function Capitalist:UpdateResourceCosts()
	self:DebugEntry("UpdateResourceCosts")

	local t

	-- populate resource costs
	self.options.args.resourceCosts.args = {}
	t = self.options.args.resourceCosts.args
	local resourceList = self:GetVisibleResources()
	table.sort(resourceList)
	t["Min"] = {
		name = "Buy price",
		type = "description",
		order = 1,
		width = "half",
	}
	t["Max"] = {
		name = "Assumed Cost",
		type = "description",
		order = 2,
		width = "half",
	}

	for i = 1, #resourceList do
		local resourceName = resourceList[i]
		t[resourceName.."Header"] = {
			name = resourceName,
			desc = nil,--string.format(L["Set the cost of %s in gold"], resourceName),
			type = "header",
			order = i * 3,
		}
		t[resourceName.."Min"] = {
			name = "",--"Max",	--resourceName,
			desc = "",--string.format(L["The production cost of resource %s in gold."], resourceName),
			type = "input",
			pattern = "^[0-9]*\.?[0-9]+$",
			order = i * 3 + 1,
			width = "half",
			-- TODO: active setting functions
			-- TODO: validate/correct if min/max flip
			set = function(info, val)
						local nVal = (tonumber(val) or 0) * 10000
						self:SetResourceCopperBuyPrice(resourceName, nVal)
						self:CostMemoryReset()
					end,
			get = function(info)
						local nVal = self:GetComponentCopperValue(resourceName) or 0
						if (self:HasResourceCopperBuyPrice(resourceName)) then
							nVal = self:GetResourceCopperBuyPrice(resourceName)
						end
						return ""..(nVal / 10000)
					end
		}
		t[resourceName.."Max"] = {
			name = "",--"Max",	--resourceName,
			desc = "",--string.format(L["The production cost of resource %s in gold."], resourceName),
			type = "input",
			pattern = "^[0-9]*\.?[0-9]+$",
			order = i * 3 + 2,
			width = "half",
			-- TODO: active setting functions
			-- TODO: validate/correct if min/max flip
			set = function(info, val)
						local nVal = (tonumber(val) or 0) * 10000
						self:SetResourceCopperCost(resourceName, nVal)
						self:CostMemoryReset()
					end,
			get = function(info)
						return ""..(self:GetResourceCopperCost(resourceName) / 10000)
					end
		}
	end

	self:DebugExit("UpdateResourceCosts")
end

function Capitalist:UpdateStatistics()
	self:DebugEntry("UpdateStatistics")

	if (self.db.profile.professions) then

		for profession, professionData in pairs(self.db.profile.professions) do

			if (self.options.args[profession]) then
				self:DebugMisc(profession)

				local t

				-- populate created products
				self.options.args.createdProducts.args = {}
				t = self.options.args.createdProducts.args

				if (self.db.profile[realm] and
					self.db.profile[realm][character] and
					self.db.profile[realm][character][profession]) then
					for skillName, skillData in pairs(self.db.profile[realm][character][profession]) do
						local numCreated = skillData.numCreated or 0

						if (professionData[skillName]) then
							local product = professionData[skillName]

							local productText = self:ProductTextFromSkillName(profession, skillName)
							if (productText and (numCreated > 0)) then
								t[skillName] = {
									name = numCreated.."x "..productText,
									type = "description",
									order = -numCreated,
								}
							end
						end
					end
				end
			end
		end
	end

	self:DebugExit("UpdateStatistics")
end
